#

/*
 *   DL-11 driver
 */

#include "../h/param.h"
#include "../h/conf.h"
#include "../h/user.h"
#include "../h/userx.h"
#include "../h/tty.h"


/* Default speed settings */

#define	B9600	015
#define	B300	07


/* Hardware info */

struct	dlregs	{	int	dlrcsr;
			int	dlrbuf;
			int	dltcsr;
			int	dltbuf;
		};

/* Bits in RCSR, RBUF, TCSR */

#define	RDRENB	01		/* Reader enable */
#define DTR	02		/* Data Terminal Rdy control */
#define	RTS	04		/* Request to Send */
#define	SECXMT	010		/* Secondary transmit */
#define	UNUS1	020
#define	DSIENBL	040		/* Data Set interrupt enable */
#define	RIENABL	0100		/* Receiver interrupt enable */
#define	RDONE	0200		/* Receiver done */
#define	UNUS2	01400
#define	SECRCV	02000		/* Secondary receive */
#define	RACT	04000		/* Receiver active */
#define	CARRDT	010000		/* Carrier detect */
#define	CTS	020000		/* Clear to send */
#define	RING	040000		/* Ring detect */
#define	DSINT	0100000		/* Data set interrupt */

#define	RCHAR	0377		/* Received data */
#define	UNUS3	07400
#define	PARERR	010000		/* Parity err */
#define	FRERR	020000		/* Framing error (break) */
#define	ORERR	040000		/* Overrun error */
#define	ERROR	0100000		/* Error detect */

#define	BRK	01		/* Break set */
#define	UNUS4	02
#define	MAINT	04		/* Maintainence (loop around */
#define	UNUS5	070
#define	TIENABL	0100		/* Transmitter interrupt enable */
#define	TDONE	0200		/* Transmitter done */
#define	UNUS6	0177400

#define	TCHAR	0377
#define	UNUS7	0177400


/* base address */

#define	DLSIZE	010			/* number of bytes for reg block */
#define	DLADDR	0177560			/* console */
#define	DLBASE	(0176500 - DLSIZE)	/* others -  minor dev nums start at 1 */


#define	NDL11	7

struct	tty dl11[NDL11];
int	ndl11	NDL11;


/* Open - see if legit, set up minor 0 to address DLADDR
 * set up minor 1 thru NDL11 to address from DLBASE.
 */


dlopen(dev, flag)

{	register	struct	dlregs	*addr;
	register	struct	tty	*tp;

	if(dev.d_minor >= NDL11) {
		u.u_error = ENXIO;
		return;
		}

	tp = &dl11[dev.d_minor];
	if (dev.d_minor == 0)
		addr = DLADDR;
	  else
		addr = DLBASE + (dev.d_minor * DLSIZE);
	tp->t_addr = addr;

	addr->dlrcsr =| RIENABL;
	addr->dltcsr =| TIENABL;

	if (!(tp->t_state & ISOPEN)) {
		tp->t_state =| CARR_ON;
		tp->t_speeds = ((B9600 << 8) | B300);
		}

	ttyopen(dev, tp);
}


/* Close, read and write let the tty routines do all the work.
 */

dlclose(dev)

{	register	struct	tty	*tp;

	tp = &dl11[dev.d_minor];
	wflushtty(tp);
	tp->t_state = 0;
}


dlread(dev)

{	ttread(&dl11[dev.d_minor]);
}


dlwrite(dev)

{	ttwrite(&dl11[dev.d_minor]);
}


dlxint(dev)

{	register	struct	tty	*tp;

	tp = &dl11[dev.d_minor];
	ttstart(tp);
	if ((tp->t_outq.c_cc == 0) || (tp->t_outq.c_cc == TTLOWAT))
		wakeup(&tp->t_outq);
}


dlrint(dev)

{	register	struct	dlregs	*addr;
	register	struct	tty	*tp;
	register	c;

	tp = &dl11[dev.d_minor];
	addr = tp->t_addr;
	c = addr->dlrbuf;

	if (c & PARERR)
		if ((tp->t_flags & (EVENP | ODDP)) != (EVENP | ODDP))
			return;
	if (c & FRERR) {
		signal(tp->t_pgrp, SIGINT);
		return;
		}
	ttyinput((c & RCHAR), tp);
}


/* dlsgtty and dlttymd basically call the right tty routine.
 * dlsgtty also has to hack the break stuff.
 */

dlsgtty(dev, v)
int	*v;

{	register	struct	tty	*tp;
	register	int	*mode;

	tp = &dl11[dev.d_minor];
	mode = u.u_arg;

	if ((v == 0) && (mode[0].lobyte == 15))
		switch (mode[0].hibyte) {
		  case 1:	tp->t_addr->dltcsr =| BRK;	/* Set break */
				return;

		  case 2:	tp->t_addr->dltcsr =& ~BRK;	/* Clear brk */
				return;
		  }

	ttystty(tp, v);
}


dlttymd(dev, acp)

{	register	struct	tty	*tp;
	tp = &dl11[dev.d_minor];

	ttmode(tp, acp);
}

dlioctl(dev, ip, cmd, cmdarg)
int	dev;
char	*ip;
int	cmd;
char	*cmdarg;
{
	register struct tty	*tp;

	tp = &dl11[dev.d_minor];
	ttioctl(tp, cmd, cmdarg);
}


