#

/*
 *   Driver for Terminet printer on DL11
 */

#include "../h/param.h"
#include "../h/user.h"
#include "../h/userx.h"


/* Say what a clist is */

struct	clist	{	int	c_cc;
			int	*c_cf;
			int	*c_cl;
		};


/* Some hardware information - location, registers, bits. */

#define	TLPADDR	0177510

struct	tlpregs	{	int	tlpsr;	/* Status register */
			int	tlpunus;
			int	tlpcr;	/* Control register */
			int	tlpbuf;	/* Buffer */
		};

#define	UNUS1	01777
#define	ONLINE	02000			/* Ready to operate */
#define	UNUS2	04000
#define	BFULL	010000			/* Buffer full - can't take chars */
#define	UNUS3	0160000

#define	UNUS4	077
#define	IENABLE	0100			/* Interrupt enable */
#define	DONE	0200			/* Character taken */
#define	UNUS5	0177400


/* Priorities, and waiting time, also water marks */

#define	TLPPO	40		/* Priority for coming online/flushing */
#define	TLPPF	10
#define	TLPTO	240		/* Waiting times */
#define	TLPTR	20
#define	TLPLWAT	30
#define	TLPHWAT	150


/* State word, clist and flags */

#define	OPEN	01
#define	CLOSING	02
#define	BUSY	04

int		tlpstate;
struct	clist	tlpq;



/* Open - mark as open and wait for it to come online */

tlpopen(dev, flag)

{	extern	lbolt;

	if (tlpstate & OPEN) {
		u.u_error = EBUSY;
		return;
		}

	tlpstate =| OPEN;

	while (!(TLPADDR->tlpsr & ONLINE))
		sleep(&lbolt, TLPPO);

	TLPADDR->tlpcr = IENABLE;
}


/* Close - wait for it to finish whats been given to use and then
 * die off. Interlock is to prevent last character draining before
 * we sleep.
 */

tlpclose(dev, flag)

{	spl4();
	while (tlpq.c_cc != 0) {
		tlpstate =| CLOSING;
		sleep(&tlpq, TLPPF);
		}
	spl0();

	TLPADDR->tlpcr = 0;
	tlpstate = 0;
}


/* While he has characters, stick them on the clist until HIWAT,
 * sleep for drain if that happens, and call start.
 */

tlpwrite()

{	register	c;

	while ((c = cpass()) >= 0) {
		if (tlpq.c_cc >= TLPHWAT)
			sleep(&tlpq, TLPPO);
		putc(c, &tlpq);
		if (!(tlpstate & BUSY))
			tlpstart();
		}
}


/* If not, if printer not ready for characters, arrange for the system
 * to try later otherwise give it to him.
 */

tlpstart()

{	extern	tlpint();

	tlpstate =| BUSY;

	if (!(TLPADDR->tlpsr & ONLINE)) {
		timeout(tlpint, 0, TLPTO);
		return;
		}
	if (TLPADDR->tlpsr & BFULL) {
		timeout(tlpint, 0, TLPTR);
		return;
		}

	TLPADDR->tlpbuf = getc(&tlpq);
}


/* If no more characters, turn printer off and wake anyone.
 * Otherwise send out a char. Wake him if we went past the low
 * water mark.
 */

tlpint()

{	if (tlpq.c_cc == 0) {
		tlpstate =& ~BUSY;
		if (tlpstate & CLOSING)
			wakeup(&tlpq);
		return;
		}

	tlpstart();
	if (tlpq.c_cc == TLPLWAT)
		wakeup(&tlpq);
}
