/*
 * The user structure.
 * One allocated per process.
 * Contains all per process data
 * that doesn't need to be referenced
 * while the process is swapped.
 * The user block is USIZE*64 bytes
 * long; resides at virtual kernel
 * loc 140000; contains the system
 * stack per user; is cross referenced
 * with the proc structure for the
 * same process.
 */
struct user
{                       /* The offsets assume DIRSIZ=14,NOFILE=20,NSIG=32 */
	int     u_rsav[2];              /* +   0=0   save r5,r6 when exchanging stacks */
	int     u_fsav[25];             /* +  04=4   save fp registers */
					/*           rsav and fsav must be first in structure */
	char    u_segflg;               /* + 066=54  IO flag: 0:user D; 1:system; 2:user I */
	char    u_error;                /* +0067=55  return error code */
	char    u_uid;                  /* +0070=56  effective user id */
	char    u_gid;                  /* +0071=57  effective group id */
	char    u_ruid;                 /* +0072=58  real user id */
	char    u_rgid;                 /* +0073=59  real group id */
	int     u_procp;                /* +0074=60  pointer to proc structure */
	char    *u_base;                /* +0076=62  base address for IO */
	char    *u_count;               /* +0100=64  bytes remaining for IO */
	char    *u_offset[2];           /* +0102=66  offset in file for IO */
	int     *u_cdir;                /* +0106=70  pointer to inode of current directory */
	char    u_dbuf[DIRSIZ];         /* +0110=72  current pathname component */
	char    *u_dirp;                /* +0126=86  current pointer to inode */
	struct  {                       /* +0130=88  current directory entry */
		int     u_ino;          /* +0130=88         */
		char    u_name[DIRSIZ]; /* +0132=90         */
	} u_dent;
	int     *u_pdir;                /* +0150=104 inode of parent directory of dirp */
	int     u_uisa[16];             /* +0152=106 prototype of segmentation addresses */
	int     u_uisd[16];             /* +0212=138 prototype of segmentation descriptors */
	int     u_ofile[NOFILE];        /* +0252=170 pointers to file structures of open files */
	int     u_arg[5];               /* +0322=210 arguments to current system call */
	int     u_tsize;                /* +0334=220 next size (*64) */
	int     u_dsize;                /* +0336=222 data size (*64) */
	int     u_ssize;                /* +0340=224 stack size (*64) */
	int     u_sep;                  /* +0342=226 flag for I and D separation */
	int     u_qsav[2];              /* +0344=228 label variable for quits and interrupts */
	int     u_ssav[2];              /* +0350=232 label variable for swapping */
	int     u_signal[NSIG+1];       /* +0354=236 disposition of signals */
	long    u_utime;                /* +0456=302 this process user time */
	long    u_stime;                /* +0462=306 this process system time */
	long    u_cutime;               /* +0466=310 sum of childs' utimes */
	long    u_cstime;               /* +0472=314 sum of childs' stimes */
	int     *u_ar0;                 /* +0476=318 address of users saved R0 */
	int     u_prof[4];              /* +0480=320 profile arguments */
	char    u_intflg;               /* +0510=328 catch intr from sys */
	int     u_ttyp;                 /* +0512=330 controlling tty pointer */
	int     u_ttyd;                 /* +0514=332 controlling tty dev */
	struct {                        /* +0516=334 header of executable file */
		int     ux_mag;         /* +0516=334 magic number */
		int     ux_tsize;       /* +0520=336 text size */
		int     ux_dsize;       /* +0522=338 data size */
		int     ux_bsize;       /* +0524=340 bss size */
		int     ux_ssize;       /* +0526=342 symbol table size */
		int     ux_entloc;      /* +0530=344 entry location */
	} u_exdata;
	char    u_comm[DIRSIZ];         /* +0532=346 */
	long    u_start;                /* +0550=360 */
	char    u_acflag;               /* +0554=364 */
	long    u_astart;               /* +0556=366 */
	long    u_sstart;               /* +0562=370 */
	long    u_ustart;               /* +0566=374 */
					/*    kernel stack per user
					 * extends from u + USIZE*64
					 * backward not to reach here
					 */
};

/* u_error codes */
#define EPERM   1
#define	ENOENT	2
#define	ESRCH	3
#define	EINTR	4
#define	EIO	5
#define	ENXIO	6
#define	E2BIG	7
#define	ENOEXEC	8
#define	EBADF	9
#define	ECHILD	10
#define	EAGAIN	11
#define	ENOMEM	12
#define	EACCES	13
#define	EFAULT	14
#define	ENOTBLK	15
#define	EBUSY	16
#define	EEXIST	17
#define	EXDEV	18
#define	ENODEV	19
#define	ENOTDIR	20
#define	EISDIR	21
#define	EINVAL	22
#define	ENFILE	23
#define	EMFILE	24
#define	ENOTTY	25
#define	ETXTBSY	26
#define	EFBIG	27
#define	ENOSPC	28
#define	ESPIPE	29
#define	EROFS	30
#define	EMLINK	31
#define	EPIPE	32
