#
#include "../h/param.h"
#include "../h/systm.h"
#include "../h/user.h"
#include "../h/userx.h"
#include "../h/proc.h"
#include "../h/procx.h"

#define	UMODE	0170000

/* Threshold for considering user CPU time `nice' */

#define	NICE	4


int	schA	0;		/*jp  Parameters for Computing Priority  */
int	schB	32;		/*jp  pri = cpu * (A/64) +
					log2(cpu) * (B/16) + nice + PUSER;  */

int	schC	52,		/*jp  Parameters for Decaying cpu-Factor  */
	schD	0;		/*jp  cpu = cpu * (C/64) - D;  */

int	schE	25;		/*rhh Param for priority of disk-bound procs */

int	schF	20,		/*rhh Params for computing current quantum */
	schG	3;		/*rhh tix = (curpri >= PUSER) ? F : G; */

/*
 * clock is called straight from
 * the real time clock interrupt.
 *
 * Functions:
 *	reprime clock
 *	copy *switches to display
 *	implement callouts
 *	maintain user/system times
 *	maintain date
 *	profile
 *	tout wakeup (sys sleep)
 *	lightning bolt wakeup (every four seconds)
 *	alarm clock signals
 *	jab the scheduler
 *	compute next quantum
 */

extern	waitloc;
char	*await { &waitloc };

clock(dev, sp, r1, nps, r0, pc, ps)
{
	register struct callo *p1, *p2;
	register struct proc *pp;
	int a;

	/*
	 * restart clock
	 */

	*lks = 0115;

	/*
	 * display register
	 */

	display();

	/*
	 * callouts
	 * if none, just return
	 * else update first non-zero time
	 */

	if(callout[0].c_func == 0)
		goto out;
	p2 = &callout[0];
	while(p2->c_time<=0 && p2->c_func!=0)
		p2++;
	p2->c_time--;

	/*
	 * if ps is high, just return
	 */

	if((ps&0340) != 0)
		goto out;

	/*
	 * callout
	 */

	spl5();
	if(callout[0].c_time <= 0) {
		p1 = &callout[0];
		while(p1->c_func != 0 && p1->c_time <= 0) {
			(*p1->c_func)(p1->c_arg);
			p1++;
		}
		p2 = &callout[0];
		while(p2->c_func = p1->c_func) {
			p2->c_time = p1->c_time;
			p2->c_arg = p1->c_arg;
			p1++;
			p2++;
		}
	}

	/*
	 * lightning bolt time-out
	 * and time of day
	 */

out:
	a = dk_busy&017;
	if((ps&UMODE) == UMODE) {
		u.u_utime++;
		if(u.u_prof[3])
			incupc(pc, u.u_prof);
		if(u.u_procp->p_nice >= NICE)
			a =+ 16;
	} else {
		a =+ 32;
		if (pc == await)
			a =+ 16;
		u.u_stime++;
	}
	dk_time[a] =+ 1;
	pp = u.u_procp;
	if(++pp->p_cpu == 0)
		pp->p_cpu--;
	if(++lbolt >= HZ)
		if((ps&0340) == 0) {
			lbolt =- HZ;
			++time;
			spl1();
			if (time == tout)
				wakeup(&tout);
			if((time.loword&03) == 0) {
				runrun++;
				wakeup(&lbolt);
			}
			for(pp = &proc[0]; pp < &proc[NPROC]; pp++) {
				if(pp->p_time != 127)
					pp->p_time++;
				if(pp->p_stat==SSLEEP || pp->p_stat==SSTOP)
					if(++pp->p_sleep == 0)
						pp->p_sleep--;
				pp->p_cpu = ((pp->p_cpu&0377) * schC) >> 6;
				if((pp->p_cpu&0377)>schD)
					pp->p_cpu =- schD;
				else
					pp->p_cpu = 0;
				if (pp->p_stat && pp->p_stat<SZOMB) {
					if(pp->p_clktim)
						if(--pp->p_clktim == 0)
							psignal(pp, SIGCLK);
				}
			}
			if(runin!=0) {
				runin = 0;
				wakeup(&runin);
			}
		}
	if(++quantum >= tix && (ps&0340) == 0) {
		for(pp = &proc[0]; pp < &proc[NPROC]; pp++)
			if(pp->p_stat && pp->p_pri > PUSER)
				setpri(pp);
		if((ps&UMODE) == UMODE) {
			u.u_ar0 = &r0;
			if(issig())
				psig();
			pp = u.u_procp;
			if (!(pp->p_flag & NOBUMP) && pp->p_nice<NICE
				&& u.u_utime > 60*HZ)
			  {	pp->p_nice =+ NICE;
				pp->p_flag =| NOBUMP;
			  }
			setpri(pp);
		}
		quantum = 0;
	}
}

/*
 * timeout is called to arrange that
 * fun(arg) is called in tim/HZ seconds.
 * An entry is sorted into the callout
 * structure. The time in each structure
 * entry is the number of HZ's more
 * than the previous entry.
 * In this way, decrementing the
 * first entry has the effect of
 * updating all entries.
 *
 * The panic is there because there is nothing
 * intelligent to be done if an entry won't fit.
 */
timeout(fun, arg, tim)
{
	register struct callo *p1, *p2;
	register t;
	int s;

	t = tim;
	s = PS->integ;
	p1 = &callout[0];
	spl7();
	while(p1->c_func != 0 && p1->c_time <= t) {
		t =- p1->c_time;
		p1++;
	}
	if (p1 >= &callout[NCALL-1])
		panic("Timeout table overflow");
	p1->c_time =- t;
	p2 = p1;
	while(p2->c_func != 0)
		p2++;
	while(p2 >= p1) {
		(p2+1)->c_time = p2->c_time;
		(p2+1)->c_func = p2->c_func;
		(p2+1)->c_arg = p2->c_arg;
		p2--;
	}
	p1->c_time = t;
	p1->c_func = fun;
	p1->c_arg = arg;
	PS->integ = s;
}
